import { useRef, useState } from 'react';

interface SelectValue {
  [unknownPropName: string]: string;
}
type FormElementType = 'input' | 'select' | 'checkbox' | 'round';
type FormElementValue = string | boolean | number | SelectValue;
type ValidationTrigger = 'change' | 'blur' | 'focus';

type SetterFunction = (value: FormElementValue) => void;
type OnSuccessCallback = () => void;
type OnErrorCallback = () => void;
type ValidationFunction = (value: FormElementValue) => Reason | Cleared;

interface Reason {
  type: keyof ValidationRules;
  message: string;
  currentValue: FormElementValue;
}

interface Cleared {
  isValid: boolean;
}

interface MinMaxRules {
  value: number;
  message?: string;
}

export interface ValidationRules {
  maxLength?: MinMaxRules;
  minLength?: MinMaxRules;
  maxValue?: MinMaxRules;
  minValue?: MinMaxRules;
  validate?: {
    [functionName: string]: {
      function: (value: FormElementType) => boolean;
      message?: string;
    };
  };
}

export type FormHandler = (
  name: string,
  type: FormElementType,
  setter: SetterFunction,
  validationFunction?: ValidationFunction
) => void;

interface FormData {
  [name: string]: {
    readonly type: FormElementType;
    readonly validationRules?: ValidationRules;
    readonly validationTrigger?: ValidationTrigger;
    readonly setter: SetterFunction;
    readonly validationFunction: ValidationFunction;
    value?: FormElementValue;
  };
}

interface ElementState {
  [name: string]: Reason | Cleared;
}

export function useFormHandler() {
  const formData = useRef({} as FormData);
  const [elementState, setElementState] = useState({} as ElementState);

  function formHandler<FormHandler>(
    name: string,
    type: FormElementType,
    setter: SetterFunction,
    validationFunction: ValidationFunction
  ) {
    formData.current[name] = { type, setter, validationFunction };
  }

  function setValue(name: string, value: FormElementValue) {
    formData.current[name].value = value;
  }

  function getValue(name: string) {
    return formData.current[name].value;
  }

  function checkValidation(name: string) {
    const { validationFunction, value } = formData.current[name];
    if (!value) {
      return false;
    }
    return validationFunction(value);
  }

  function checkAllElements() {
    const names = Object.keys(formData.current);
    const revalidationErrors = elementState;
    names.forEach((name) => {
      const elementState = checkValidation(name);
      // revalidationErrors[name] = elementState;
    });
    setElementState(revalidationErrors);
  }

  function handleSubmit(
    onSuccess: OnSuccessCallback,
    onError: OnErrorCallback
  ) {}

  return {
    formHandler
  };
}
